@@ -44,6 +44,10 @@ def handle_item(config: dict, item: str, data: dict, used_textures: set|None = N
 		content = {}
 		if data.get(OVERRIDE_MODEL):
 			content = data[OVERRIDE_MODEL]
+			if not content.get("textures"):
+				new_content = {"textures": {"layer0": f"{config['namespace']}:{block_or_item}/{item}{on_off}"}}
+				new_content.update(content)
+				content = new_content
 			if on_off:	# Check if the override model has the on/off texture
 				for key, texture in content["textures"].items():
 					on_off_path = "/".join(texture.split("/")[1:]) + on_off + ".png"
